@@ -18,20 +18,30 @@ class AvroProducer(Producer):
         Kafka Producer client which does avro schema encoding to messages.
         Handles schema registration, Message serialization.
 
-        Constructor takes below parameters
+        Constructor takes below parameters.
 
-        @:param: config: dict object with config parameters containing url for schema registry (schema.registry.url).
-        @:param: default_key_schema: Optional avro schema for key
-        @:param: default_value_schema: Optional avro schema for value
+        :param dict config: Config parameters containing url for schema registry (``schema.registry.url``)
+                            and the standard Kafka client configuration (``bootstrap.servers`` et.al).
+        :param str default_key_schema: Optional default avro schema for key
+        :param str default_value_schema: Optional default avro schema for value
     """
 
     def __init__(self, config, default_key_schema=None,
                  default_value_schema=None, schema_registry=None):
+
         schema_registry_url = config.pop("schema.registry.url", None)
+        schema_registry_ca_location = config.pop("schema.registry.ssl.ca.location", None)
+        schema_registry_certificate_location = config.pop("schema.registry.ssl.certificate.location", None)
+        schema_registry_key_location = config.pop("schema.registry.ssl.key.location", None)
+
         if schema_registry is None:
             if schema_registry_url is None:
                 raise ValueError("Missing parameter: schema.registry.url")
-            schema_registry = CachedSchemaRegistryClient(url=schema_registry_url)
+
+            schema_registry = CachedSchemaRegistryClient(url=schema_registry_url,
+                                                         ca_location=schema_registry_ca_location,
+                                                         cert_location=schema_registry_certificate_location,
+                                                         key_location=schema_registry_key_location)
         elif schema_registry_url is not None:
             raise ValueError("Cannot pass schema_registry along with schema.registry.url config")
 
@@ -42,13 +52,19 @@ class AvroProducer(Producer):
 
     def produce(self, **kwargs):
         """
-            Sends message to kafka by encoding with specified avro schema
-            @:param: topic: topic name
-            @:param: value: An object to serialize
-            @:param: value_schema : Avro schema for value
-            @:param: key: An object to serialize
-            @:param: key_schema : Avro schema for key
-            @:exception: SerializerError
+            Asynchronously sends message to Kafka by encoding with specified or default avro schema.
+
+            :param str topic: topic name
+            :param object value: An object to serialize
+            :param str value_schema: Avro schema for value
+            :param object key: An object to serialize
+            :param str key_schema: Avro schema for key
+
+            Plus any other parameters accepted by confluent_kafka.Producer.produce
+
+            :raises SerializerError: On serialization failure
+            :raises BufferError: If producer queue is full.
+            :raises KafkaException: For other produce failures.
         """
         # get schemas from  kwargs if defined
         key_schema = kwargs.pop('key_schema', self._key_schema)
@@ -59,13 +75,13 @@ class AvroProducer(Producer):
         value = kwargs.pop('value', None)
         key = kwargs.pop('key', None)
 
-        if value:
+        if value is not None:
             if value_schema:
                 value = self._serializer.encode_record_with_schema(topic, value_schema, value)
             else:
                 raise ValueSerializerError("Avro schema required for values")
 
-        if key:
+        if key is not None:
             if key_schema:
                 key = self._serializer.encode_record_with_schema(topic, key_schema, key, True)
             else:
@@ -81,14 +97,24 @@ class AvroConsumer(Consumer):
 
     Constructor takes below parameters
 
-    @:param: config: dict object with config parameters containing url for schema registry (schema.registry.url).
+    :param dict config: Config parameters containing url for schema registry (``schema.registry.url``)
+                        and the standard Kafka client configuration (``bootstrap.servers`` et.al).
     """
     def __init__(self, config, schema_registry=None):
+
         schema_registry_url = config.pop("schema.registry.url", None)
+        schema_registry_ca_location = config.pop("schema.registry.ssl.ca.location", None)
+        schema_registry_certificate_location = config.pop("schema.registry.ssl.certificate.location", None)
+        schema_registry_key_location = config.pop("schema.registry.ssl.key.location", None)
+
         if schema_registry is None:
             if schema_registry_url is None:
                 raise ValueError("Missing parameter: schema.registry.url")
-            schema_registry = CachedSchemaRegistryClient(url=schema_registry_url)
+
+            schema_registry = CachedSchemaRegistryClient(url=schema_registry_url,
+                                                         ca_location=schema_registry_ca_location,
+                                                         cert_location=schema_registry_certificate_location,
+                                                         key_location=schema_registry_key_location)
         elif schema_registry_url is not None:
             raise ValueError("Cannot pass schema_registry along with schema.registry.url config")
 
@@ -100,8 +126,9 @@ class AvroConsumer(Consumer):
         This is an overriden method from confluent_kafka.Consumer class. This handles message
         deserialization using avro schema
 
-        @:param timeout
-        @:return message object with deserialized key and value as dict objects
+        :param float timeout: Poll timeout in seconds (default: indefinite)
+        :returns: message object with deserialized key and value as dict objects
+        :rtype: Message
         """
         if timeout is None:
             timeout = -1
