@@ -29,6 +29,7 @@ from .error import ClientError
 from . import loads
 
 VALID_LEVELS = ['NONE', 'FULL', 'FORWARD', 'BACKWARD']
+VALID_METHODS = ['GET', 'POST', 'PUT', 'DELETE']
 
 # Common accept header sent
 ACCEPT_HDR = "application/vnd.schemaregistry.v1+json, application/vnd.schemaregistry+json, application/json"
@@ -46,7 +47,7 @@ class CachedSchemaRegistryClient(object):
     @:param: url: url to schema registry
     """
 
-    def __init__(self, url, max_schemas_per_subject=1000):
+    def __init__(self, url, max_schemas_per_subject=1000, ca_location=None, cert_location=None, key_location=None):
         """Construct a client by passing in the base URL of the schema registry server"""
 
         self.url = url.rstrip('/')
@@ -59,32 +60,41 @@ class CachedSchemaRegistryClient(object):
         # subj => { schema => version }
         self.subject_to_schema_versions = defaultdict(dict)
 
-    def _send_request(self, url, method='GET', body=None, headers=None):
-        if body:
-            body = json.dumps(body)
-            body = body.encode('utf8')
-        _headers = dict()
-        _headers["Accept"] = ACCEPT_HDR
+        s = requests.Session()
+        if ca_location is not None:
+            s.verify = ca_location
+        if cert_location is not None or key_location is not None:
+            if cert_location is None or key_location is None:
+                raise ValueError(
+                    "Both schema.registry.ssl.certificate.location and schema.registry.ssl.key.location must be set")
+            s.cert = (cert_location, key_location)
+
+        self._session = s
+
+    def __del__(self):
+        self.close()
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, *args):
+        self.close()
+
+    def close(self):
+        self._session.close()
+
+    def _send_request(self, url, method='GET', body=None, headers={}):
+        if method not in VALID_METHODS:
+            raise ClientError("Method {} is invalid; valid methods include {}".format(method, VALID_METHODS))
+
+        _headers = {'Accept': ACCEPT_HDR}
         if body:
             _headers["Content-Length"] = str(len(body))
             _headers["Content-Type"] = "application/vnd.schemaregistry.v1+json"
+        _headers.update(headers)
 
-        if headers:
-            for header_name in headers:
-                _headers[header_name] = headers[header_name]
-        if method == 'GET':
-            response = requests.get(url, headers=_headers)
-        elif method == 'POST':
-            response = requests.post(url, body, headers=_headers)
-        elif method == 'PUT':
-            response = requests.put(url, body, headers=_headers)
-        elif method == 'DELETE':
-            response = requests.delete(url, headers=_headers)
-        else:
-            raise ClientError("Invalid HTTP request type")
-
-        result = json.loads(response.text)
-        return (result, response.status_code)
+        response = self._session.request(method, url, headers=_headers, json=body)
+        return response.json(), response.status_code
 
     def _add_to_cache(self, cache, subject, schema, value):
         sub_cache = cache[subject]
@@ -168,9 +178,9 @@ class CachedSchemaRegistryClient(object):
                 # cache it
                 self._cache_schema(result, schema_id)
                 return result
-            except:
+            except ClientError as e:
                 # bad schema - should not happen
-                raise ClientError("Received bad schema from registry.")
+                raise ClientError("Received bad schema (id %s) from registry: %s" % (schema_id, e))
 
     def get_latest_schema(self, subject):
         """
@@ -204,9 +214,9 @@ class CachedSchemaRegistryClient(object):
         else:
             try:
                 schema = loads(result['schema'])
-            except:
+            except ClientError:
                 # bad schema - should not happen
-                raise ClientError("Received bad schema from registry.")
+                raise
 
         self._cache_schema(schema, schema_id, subject, version)
         return (schema_id, schema, version)
@@ -267,9 +277,10 @@ class CachedSchemaRegistryClient(object):
             elif code >= 200 and code <= 299:
                 return result.get('is_compatible')
             else:
-                log.error("Unable to check the compatibility")
-                False
-        except:
+                log.error("Unable to check the compatibility: " + str(code))
+                return False
+        except Exception as e:
+            log.error("_send_request() failed: %s", e)
             return False
 
     def update_compatibility(self, level, subject=None):
@@ -300,6 +311,7 @@ class CachedSchemaRegistryClient(object):
         Get the current compatibility level for a subject.  Result will be one of:
 
         @:param: subject: subject name
+        @:raises: ClientError: if the request was unsuccessful or an invalid compatibility level was returned
         @:return: 'NONE','FULL','FORWARD', or 'BACKWARD'
         """
         url = '/'.join([self.url, 'config'])
@@ -307,10 +319,16 @@ class CachedSchemaRegistryClient(object):
             url += '/' + subject
 
         result, code = self._send_request(url)
-        if code >= 200 and code <= 299:
-            compatibility = result.get('compatibility', None)
+        is_successful_request = code >= 200 and code <= 299
+        if not is_successful_request:
+            raise ClientError('Unable to fetch compatibility level. Error code: %d' % code)
 
-        if not compatibility:
-            compatibility = result.get('compatibilityLevel')
+        compatibility = result.get('compatibility', None)
+        if compatibility not in VALID_LEVELS:
+            if compatibility is None:
+                error_msg_suffix = 'No compatibility was returned'
+            else:
+                error_msg_suffix = str(compatibility)
+            raise ClientError('Invalid compatibility level received: %s' % error_msg_suffix)
 
         return compatibility
