@@ -48,7 +48,6 @@
 struct Producer_msgstate {
 	Handle   *self;
 	PyObject *dr_cb;
-	PyObject *partitioner_cb;
 };
 
 
@@ -58,12 +57,9 @@ struct Producer_msgstate {
  */
 static __inline struct Producer_msgstate *
 Producer_msgstate_new (Handle *self,
-		       PyObject *dr_cb, PyObject *partitioner_cb) {
+		       PyObject *dr_cb) {
 	struct Producer_msgstate *msgstate;
 
-	if (!dr_cb && !partitioner_cb)
-		return NULL;
-
 	msgstate = calloc(1, sizeof(*msgstate));
 	msgstate->self = self;
 
@@ -71,10 +67,6 @@ Producer_msgstate_new (Handle *self,
 		msgstate->dr_cb = dr_cb;
 		Py_INCREF(dr_cb);
 	}
-	if (partitioner_cb) {
-		msgstate->partitioner_cb = partitioner_cb;
-		Py_INCREF(partitioner_cb);
-	}
 	return msgstate;
 }
 
@@ -82,33 +74,29 @@ static __inline void
 Producer_msgstate_destroy (struct Producer_msgstate *msgstate) {
 	if (msgstate->dr_cb)
 		Py_DECREF(msgstate->dr_cb);
-	if (msgstate->partitioner_cb)
-		Py_DECREF(msgstate->partitioner_cb);
 	free(msgstate);
 }
 
 
+static void Producer_clear0 (Handle *self) {
+        if (self->u.Producer.default_dr_cb) {
+                Py_DECREF(self->u.Producer.default_dr_cb);
+                self->u.Producer.default_dr_cb = NULL;
+        }
+}
+
 static int Producer_clear (Handle *self) {
-	if (self->u.Producer.default_dr_cb) {
-		Py_DECREF(self->u.Producer.default_dr_cb);
-		self->u.Producer.default_dr_cb = NULL;
-	}
-	if (self->u.Producer.partitioner_cb) {
-		Py_DECREF(self->u.Producer.partitioner_cb);
-		self->u.Producer.partitioner_cb = NULL;
-	}
-
-	Handle_clear(self);
-
-	return 0;
+        Producer_clear0(self);
+        Handle_clear(self);
+        return 0;
 }
 
 static void Producer_dealloc (Handle *self) {
 	PyObject_GC_UnTrack(self);
 
-	Producer_clear(self);
+        Producer_clear0(self);
 
-	if (self->rk) {
+        if (self->rk) {
                 CallState cs;
                 CallState_begin(self, &cs);
 
@@ -117,6 +105,8 @@ static void Producer_dealloc (Handle *self) {
                 CallState_end(self, &cs);
         }
 
+        Handle_clear(self);
+
 	Py_TYPE(self)->tp_free((PyObject *)self);
 }
 
@@ -124,8 +114,6 @@ static int Producer_traverse (Handle *self,
 			      visitproc visit, void *arg) {
 	if (self->u.Producer.default_dr_cb)
 		Py_VISIT(self->u.Producer.default_dr_cb);
-	if (self->u.Producer.partitioner_cb)
-		Py_VISIT(self->u.Producer.partitioner_cb);
 
 	Handle_traverse(self, visit, arg);
 
@@ -187,78 +175,17 @@ static void dr_msg_cb (rd_kafka_t *rk, const rd_kafka_message_t *rkm,
 }
 
 
-/**
- * FIXME: The partitioner is currently broken due to threading/GIL issues.
- */
-int32_t Producer_partitioner_cb (const rd_kafka_topic_t *rkt,
-				 const void *keydata,
-				 size_t keylen,
-				 int32_t partition_cnt,
-				 void *rkt_opaque, void *msg_opaque) {
-	Handle *self = rkt_opaque;
-	struct Producer_msgstate *msgstate = msg_opaque;
-	PyGILState_STATE gstate;
-	PyObject *result;
-	PyObject *args;
-	int32_t r = RD_KAFKA_PARTITION_UA;
-
-	if (!msgstate) {
-		/* Fall back on default C partitioner if neither a per-msg
-		 * partitioner nor a default Python partitioner is available */
-		return self->u.Producer.c_partitioner_cb(rkt, keydata, keylen,
-							 partition_cnt,
-							 rkt_opaque, msg_opaque);
-	}
-
-	gstate = PyGILState_Ensure();
-
-	if (!msgstate->partitioner_cb) {
-		/* Fall back on default C partitioner if neither a per-msg
-		 * partitioner nor a default Python partitioner is available */
-		r = msgstate->self->u.Producer.c_partitioner_cb(rkt,
-								keydata, keylen,
-								partition_cnt,
-								rkt_opaque,
-								msg_opaque);
-		goto done;
-	}
-
-	args = Py_BuildValue("(s#l)",
-			     (const char *)keydata, (int)keylen,
-			     (long)partition_cnt);
-	if (!args) {
-		cfl_PyErr_Format(RD_KAFKA_RESP_ERR__FAIL,
-				 "Unable to build callback args");
-		goto done;
-	}
-
-
-	result = PyObject_CallObject(msgstate->partitioner_cb, args);
-	Py_DECREF(args);
-
-	if (result) {
-		r = (int32_t)PyLong_AsLong(result);
-		if (PyErr_Occurred())
-			printf("FIXME: partition_cb returned wrong type "
-			       "(expected long), how to propagate?\n");
-		Py_DECREF(result);
-	} else {
-		printf("FIXME: partitioner_cb crashed, how to propagate?\n");
-	}
-
- done:
-	PyGILState_Release(gstate);
-	return r;
-}
-
-
 #if HAVE_PRODUCEV
 static rd_kafka_resp_err_t
 Producer_producev (Handle *self,
                    const char *topic, int32_t partition,
                    const void *value, size_t value_len,
                    const void *key, size_t key_len,
-                   void *opaque, int64_t timestamp) {
+                   void *opaque, int64_t timestamp
+#ifdef RD_KAFKA_V_HEADERS
+                   ,rd_kafka_headers_t *headers
+#endif
+                   ) {
 
         return rd_kafka_producev(self->rk,
                                  RD_KAFKA_V_MSGFLAGS(RD_KAFKA_MSG_F_COPY),
@@ -268,6 +195,9 @@ Producer_producev (Handle *self,
                                  RD_KAFKA_V_VALUE((void *)value,
                                                   (size_t)value_len),
                                  RD_KAFKA_V_TIMESTAMP(timestamp),
+#ifdef RD_KAFKA_V_HEADERS
+                                 RD_KAFKA_V_HEADERS(headers),
+#endif
                                  RD_KAFKA_V_OPAQUE(opaque),
                                  RD_KAFKA_V_END);
 }
@@ -302,27 +232,31 @@ static PyObject *Producer_produce (Handle *self, PyObject *args,
 	const char *topic, *value = NULL, *key = NULL;
 	int value_len = 0, key_len = 0;
 	int partition = RD_KAFKA_PARTITION_UA;
-	PyObject *dr_cb = NULL, *dr_cb2 = NULL, *partitioner_cb = NULL;
+	PyObject *headers = NULL, *dr_cb = NULL, *dr_cb2 = NULL;
         long long timestamp = 0;
         rd_kafka_resp_err_t err;
 	struct Producer_msgstate *msgstate;
+#ifdef RD_KAFKA_V_HEADERS
+    rd_kafka_headers_t *rd_headers = NULL;
+#endif
+
 	static char *kws[] = { "topic",
 			       "value",
 			       "key",
 			       "partition",
 			       "callback",
 			       "on_delivery", /* Alias */
-			       "partitioner",
-                               "timestamp",
+                   "timestamp",
+                   "headers",
 			       NULL };
 
 	if (!PyArg_ParseTupleAndKeywords(args, kwargs,
-					 "s|z#z#iOOOL"
+					 "s|z#z#iOOLO"
                                          , kws,
 					 &topic, &value, &value_len,
 					 &key, &key_len, &partition,
-					 &dr_cb, &dr_cb2, &partitioner_cb,
-                                         &timestamp))
+					 &dr_cb, &dr_cb2,
+                     &timestamp, &headers))
 		return NULL;
 
 #if !HAVE_PRODUCEV
@@ -337,30 +271,49 @@ static PyObject *Producer_produce (Handle *self, PyObject *args,
         }
 #endif
 
+#ifndef RD_KAFKA_V_HEADERS
+    if (headers) {
+            PyErr_Format(PyExc_NotImplementedError,
+                         "Producer message headers requires "
+                         "confluent-kafka-python built for librdkafka "
+                         "version >=v0.11.4 (librdkafka runtime 0x%x, "
+                         "buildtime 0x%x)",
+                         rd_kafka_version(), RD_KAFKA_VERSION);
+            return NULL;
+    }
+#else
+    if (headers) {
+        if(!(rd_headers = py_headers_to_c(headers)))
+            return NULL;
+    }
+#endif
+
+
 	if (dr_cb2 && !dr_cb) /* Alias */
 		dr_cb = dr_cb2;
 
 	if (!dr_cb || dr_cb == Py_None)
 		dr_cb = self->u.Producer.default_dr_cb;
-	if (!partitioner_cb || partitioner_cb == Py_None)
-		partitioner_cb = self->u.Producer.partitioner_cb;
 
 	/* Create msgstate if necessary, may return NULL if no callbacks
 	 * are wanted. */
-	msgstate = Producer_msgstate_new(self, dr_cb, partitioner_cb);
+	msgstate = Producer_msgstate_new(self, dr_cb);
 
         /* Produce message */
 #if HAVE_PRODUCEV
         err = Producer_producev(self, topic, partition,
                                 value, value_len,
                                 key, key_len,
-                                msgstate, timestamp);
+                                msgstate, timestamp
+#ifdef RD_KAFKA_V_HEADERS
+                                ,rd_headers
+#endif
+                                );
 #else
         err = Producer_produce0(self, topic, partition,
                                 value, value_len,
                                 key, key_len,
                                 msgstate);
-
 #endif
 
         if (err) {
@@ -416,7 +369,7 @@ static PyObject *Producer_poll (Handle *self, PyObject *args,
 	if (r == -1)
 		return NULL;
 
-	return PyLong_FromLong(r);
+	return cfl_PyInt_FromInt(r);
 }
 
 
@@ -444,31 +397,35 @@ static PyObject *Producer_flush (Handle *self, PyObject *args,
                         return NULL;
         }
 #endif
-        return PyLong_FromLong(qlen);
+        return cfl_PyInt_FromInt(qlen);
 }
 
-
 static PyMethodDef Producer_methods[] = {
 	{ "produce", (PyCFunction)Producer_produce,
 	  METH_VARARGS|METH_KEYWORDS,
-	  ".. py:function:: produce(topic, [value], [key], [partition], [on_delivery], [timestamp])\n"
+	  ".. py:function:: produce(topic, [value], [key], [partition], [on_delivery], [timestamp], [headers])\n"
 	  "\n"
 	  "  Produce message to topic.\n"
 	  "  This is an asynchronous operation, an application may use the "
 	  "``callback`` (alias ``on_delivery``) argument to pass a function "
 	  "(or lambda) that will be called from :py:func:`poll()` when the "
 	  "message has been successfully delivered or permanently fails delivery.\n"
+      "\n"
+      "  Currently message headers are not supported on the message returned to the "
+      "callback. The ``msg.headers()`` will return None even if the original message "
+      "had headers set.\n"
 	  "\n"
 	  "  :param str topic: Topic to produce message to\n"
 	  "  :param str|bytes value: Message payload\n"
 	  "  :param str|bytes key: Message key\n"
-	  "  :param int partition: Partition to produce to, elses uses the "
-	  "configured partitioner.\n"
+	  "  :param int partition: Partition to produce to, else uses the "
+	  "configured built-in partitioner.\n"
 	  "  :param func on_delivery(err,msg): Delivery report callback to call "
 	  "(from :py:func:`poll()` or :py:func:`flush()`) on successful or "
 	  "failed delivery\n"
-          "  :param int timestamp: Message timestamp (CreateTime) in microseconds since epoch UTC (requires librdkafka >= v0.9.4, api.version.request=true, and broker >= 0.10.0.0). Default value is current time.\n"
+          "  :param int timestamp: Message timestamp (CreateTime) in milliseconds since epoch UTC (requires librdkafka >= v0.9.4, api.version.request=true, and broker >= 0.10.0.0). Default value is current time.\n"
 	  "\n"
+          "  :param headers dict|list: Message headers to set on the message. The header key must be a string while the value must be binary, unicode or None. Accepts a list of (key,value) or a dict. (Requires librdkafka >= v0.11.4 and broker version >= 0.11.0.0)\n"
 	  "  :rtype: None\n"
 	  "  :raises BufferError: if the internal producer message queue is "
 	  "full (``queue.buffering.max.messages`` exceeded)\n"
@@ -488,7 +445,7 @@ static PyMethodDef Producer_methods[] = {
 	  "  - ``on_delivery`` callbacks from :py:func:`produce()`\n"
 	  "  - ...\n"
 	  "\n"
-	  "  :param float timeout: Maximum time to block waiting for events.\n"
+	  "  :param float timeout: Maximum time to block waiting for events. (Seconds)\n"
 	  "  :returns: Number of events processed (callbacks served)\n"
 	  "  :rtype: int\n"
 	  "\n"
@@ -501,13 +458,17 @@ static PyMethodDef Producer_methods[] = {
 	  "   This is a convenience method that calls :py:func:`poll()` until "
 	  ":py:func:`len()` is zero or the optional timeout elapses.\n"
 	  "\n"
-          "  :param: float timeout: Maximum time to block (requires librdkafka >= v0.9.4).\n"
+          "  :param: float timeout: Maximum time to block (requires librdkafka >= v0.9.4). (Seconds)\n"
           "  :returns: Number of messages still in queue.\n"
           "\n"
 	  ".. note:: See :py:func:`poll()` for a description on what "
 	  "callbacks may be triggered.\n"
 	  "\n"
 	},
+        { "list_topics", (PyCFunction)list_topics, METH_VARARGS|METH_KEYWORDS,
+          list_topics_doc
+        },
+
 	{ NULL }
 };
 
@@ -550,6 +511,10 @@ static int Producer_init (PyObject *selfobj, PyObject *args, PyObject *kwargs) {
                 return -1;
         }
 
+        /* Forward log messages to poll queue */
+        if (self->logger)
+                rd_kafka_set_log_queue(self->rk, NULL);
+
         return 0;
 }
 
@@ -583,18 +548,20 @@ PyTypeObject ProducerType = {
 	0,                         /*tp_as_buffer*/
 	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE |
 	Py_TPFLAGS_HAVE_GC, /*tp_flags*/
-	"Asynchronous Kafka Producer\n"
-	"\n"
-	".. py:function:: Producer(**kwargs)\n"
-	"\n"
-	"  Create new Producer instance using provided configuration dict.\n"
-	"\n"
-	"\n"
-	".. py:function:: len()\n"
-	"\n"
-	"  :returns: Number of messages and Kafka protocol requests waiting to be delivered to broker.\n"
-	"  :rtype: int\n"
-	"\n", /*tp_doc*/
+        "Asynchronous Kafka Producer\n"
+        "\n"
+        ".. py:function:: Producer(config)\n"
+        "\n"
+        "  :param dict config: Configuration properties. At a minimum ``bootstrap.servers`` **should** be set\n"
+        "\n"
+        "  Create new Producer instance using provided configuration dict.\n"
+        "\n"
+        "\n"
+        ".. py:function:: len()\n"
+        "\n"
+        "  :returns: Number of messages and Kafka protocol requests waiting to be delivered to broker.\n"
+        "  :rtype: int\n"
+        "\n", /*tp_doc*/
 	(traverseproc)Producer_traverse, /* tp_traverse */
 	(inquiry)Producer_clear, /* tp_clear */
 	0,		           /* tp_richcompare */
