@@ -756,7 +756,7 @@ static PyObject *TopicPartition_str0 (TopicPartition *self) {
 	PyObject *ret;
 	char offset_str[40];
 
-	snprintf(offset_str, sizeof(offset_str), "%"PRId64"", self->offset);
+	snprintf(offset_str, sizeof(offset_str), "%"CFL_PRId64"", self->offset);
 
         if (self->error != Py_None) {
                 errstr = cfl_PyObject_Unistr(self->error);
@@ -764,7 +764,7 @@ static PyObject *TopicPartition_str0 (TopicPartition *self) {
         }
 
 	ret = cfl_PyUnistr(
-		_FromFormat("TopicPartition{topic=%s,partition=%"PRId32
+		_FromFormat("TopicPartition{topic=%s,partition=%"CFL_PRId32
 			    ",offset=%s,error=%s}",
 			    self->topic, self->partition,
 			    offset_str,
@@ -1001,6 +1001,7 @@ static int py_header_to_c (rd_kafka_headers_t *rd_headers,
                         if (cfl_PyBin(_AsStringAndSize(value, (char **)&v,
                                                        &vsize)) == -1) {
                                 Py_DECREF(ks);
+                                Py_XDECREF(ks8);
                                 return 0;
                         }
                 } else if (cfl_PyUnistr(_Check(value))) {
@@ -1008,6 +1009,7 @@ static int py_header_to_c (rd_kafka_headers_t *rd_headers,
                         v = cfl_PyUnistr_AsUTF8(value, &vo8);
                         if (!v) {
                                 Py_DECREF(ks);
+                                Py_XDECREF(ks8);
                                 return 0;
                         }
                         vsize = (Py_ssize_t)strlen(v);
@@ -1017,6 +1019,8 @@ static int py_header_to_c (rd_kafka_headers_t *rd_headers,
                                      "None, binary, or unicode string, not %s",
                                      ((PyTypeObject *)PyObject_Type(value))->
                                      tp_name);
+                        Py_DECREF(ks);
+                        Py_XDECREF(ks8);
                         return 0;
                 }
         }
@@ -1027,11 +1031,13 @@ static int py_header_to_c (rd_kafka_headers_t *rd_headers,
                                  "%s",
                                  k, rd_kafka_err2str(err));
                 Py_DECREF(ks);
+                Py_XDECREF(ks8);
                 Py_XDECREF(vo8);
                 return 0;
         }
 
         Py_DECREF(ks);
+        Py_XDECREF(ks8);
         Py_XDECREF(vo8);
 
         return 1;
@@ -1282,14 +1288,14 @@ static void log_cb (const rd_kafka_t *rk, int level,
         CallState *cs;
         static const int level_map[8] = {
                 /* Map syslog levels to python logging levels */
-                [0] = 50, /* LOG_EMERG   -> logging.CRITICAL */
-                [1] = 50, /* LOG_ALERT   -> logging.CRITICAL */
-                [2] = 50, /* LOG_CRIT    -> logging.CRITICAL */
-                [3] = 40, /* LOG_ERR     -> logging.ERROR */
-                [4] = 30, /* LOG_WARNING -> logging.WARNING */
-                [5] = 20, /* LOG_NOTICE  -> logging.INFO */
-                [6] = 20, /* LOG_INFO    -> logging.INFO */
-                [7] = 10, /* LOG_DEBUG   -> logging.DEBUG */
+                50, /* LOG_EMERG   -> logging.CRITICAL */
+                50, /* LOG_ALERT   -> logging.CRITICAL */
+                50, /* LOG_CRIT    -> logging.CRITICAL */
+                40, /* LOG_ERR     -> logging.ERROR */
+                30, /* LOG_WARNING -> logging.WARNING */
+                20, /* LOG_NOTICE  -> logging.INFO */
+                20, /* LOG_INFO    -> logging.INFO */
+                10, /* LOG_DEBUG   -> logging.DEBUG */
         };
 
         cs = CallState_get(h);
@@ -1324,16 +1330,25 @@ static void log_cb (const rd_kafka_t *rk, int level,
  * Clear Python object references in Handle
  */
 void Handle_clear (Handle *h) {
-	if (h->error_cb)
-		Py_DECREF(h->error_cb);
+        if (h->error_cb) {
+                Py_DECREF(h->error_cb);
+                h->error_cb = NULL;
+        }
 
-        if (h->throttle_cb)
+        if (h->throttle_cb) {
                 Py_DECREF(h->throttle_cb);
+                h->throttle_cb = NULL;
+        }
 
-	if (h->stats_cb)
-		Py_DECREF(h->stats_cb);
+        if (h->stats_cb) {
+                Py_DECREF(h->stats_cb);
+                h->stats_cb = NULL;
+        }
 
-        Py_XDECREF(h->logger);
+        if (h->logger) {
+                Py_DECREF(h->logger);
+                h->logger = NULL;
+        }
 
         if (h->initiated) {
 #ifdef WITH_PY_TSS
@@ -1360,82 +1375,15 @@ int Handle_traverse (Handle *h, visitproc visit, void *arg) {
 	return 0;
 }
 
-
-
-/**
- * Populate topic conf from provided dict.
- *
- * Will raise an exception on error and return -1, or returns 0 on success.
- */
-static int populate_topic_conf (rd_kafka_topic_conf_t *tconf, const char *what,
-				PyObject *dict) {
-	Py_ssize_t pos = 0;
-	PyObject *ko, *vo;
-
-	if (!PyDict_Check(dict)) {
-		cfl_PyErr_Format(RD_KAFKA_RESP_ERR__INVALID_ARG,
-				 "%s: requires a dict", what);
-		return -1;
-	}
-
-	while (PyDict_Next(dict, &pos, &ko, &vo)) {
-		PyObject *ks, *ks8;
-		PyObject *vs, *vs8;
-		const char *k;
-		const char *v;
-		char errstr[256];
-
-		if (!(ks = cfl_PyObject_Unistr(ko))) {
-			PyErr_SetString(PyExc_TypeError,
-					"expected configuration property "
-					"value as type unicode string");
-			return -1;
-		}
-
-		if (!(vs = cfl_PyObject_Unistr(vo))) {
-			PyErr_SetString(PyExc_TypeError,
-					"expected configuration property "
-					"value as type unicode string");
-			Py_DECREF(ks);
-			return -1;
-		}
-
-		k = cfl_PyUnistr_AsUTF8(ks, &ks8);
-		v = cfl_PyUnistr_AsUTF8(vs, &vs8);
-
-		if (rd_kafka_topic_conf_set(tconf, k, v,
-					    errstr, sizeof(errstr)) !=
-		    RD_KAFKA_CONF_OK) {
-			cfl_PyErr_Format(RD_KAFKA_RESP_ERR__INVALID_ARG,
-					 "%s: %s", what, errstr);
-                        Py_XDECREF(ks8);
-                        Py_XDECREF(vs8);
-			Py_DECREF(ks);
-			Py_DECREF(vs);
-			return -1;
-		}
-
-                Py_XDECREF(ks8);
-                Py_XDECREF(vs8);
-		Py_DECREF(ks);
-		Py_DECREF(vs);
-	}
-
-	return 0;
-}
-
-
-
 /**
  * @brief Set single special producer config value.
  *
  * @returns 1 if handled, 0 if unknown, or -1 on failure (exception raised).
  */
 static int producer_conf_set_special (Handle *self, rd_kafka_conf_t *conf,
-				      rd_kafka_topic_conf_t *tconf,
 				      const char *name, PyObject *valobj) {
 
-	if (!strcasecmp(name, "on_delivery")) {
+	if (!strcmp(name, "on_delivery")) {
 		if (!PyCallable_Check(valobj)) {
 			cfl_PyErr_Format(
 				RD_KAFKA_RESP_ERR__INVALID_ARG,
@@ -1474,10 +1422,9 @@ static int producer_conf_set_special (Handle *self, rd_kafka_conf_t *conf,
  * @returns 1 if handled, 0 if unknown, or -1 on failure (exception raised).
  */
 static int consumer_conf_set_special (Handle *self, rd_kafka_conf_t *conf,
-				      rd_kafka_topic_conf_t *tconf,
 				      const char *name, PyObject *valobj) {
 
-	if (!strcasecmp(name, "on_commit")) {
+	if (!strcmp(name, "on_commit")) {
 		if (!PyCallable_Check(valobj)) {
 			cfl_PyErr_Format(
 				RD_KAFKA_RESP_ERR__INVALID_ARG,
@@ -1495,6 +1442,80 @@ static int consumer_conf_set_special (Handle *self, rd_kafka_conf_t *conf,
 	return 0;
 }
 
+/**
+ * @brief Call out to __init__.py _resolve_plugins() to see if any
+ *        of the specified `plugin.library.paths` are found in the
+ *        wheel's embedded library directory, and if so change the
+ *        path to use these libraries.
+ *
+ * @returns a possibly updated plugin.library.paths string object which
+ *          must be DECREF:ed, or NULL if an exception was raised.
+ */
+static PyObject *resolve_plugins (PyObject *plugins) {
+        PyObject *resolved;
+        PyObject *module, *function;
+
+        module = PyImport_ImportModule("confluent_kafka");
+        if (!module)
+                return NULL;
+
+        function = PyObject_GetAttrString(module, "_resolve_plugins");
+        if (!function) {
+                PyErr_SetString(PyExc_RuntimeError,
+                                "confluent_kafka._resolve_plugins() not found");
+                Py_DECREF(module);
+                return NULL;
+        }
+
+        resolved = PyObject_CallFunctionObjArgs(function, plugins, NULL);
+
+        Py_DECREF(function);
+        Py_DECREF(module);
+
+        if (!resolved) {
+                PyErr_SetString(PyExc_RuntimeError,
+                                "confluent_kafka._resolve_plugins() failed");
+                return NULL;
+        }
+
+        return resolved;
+}
+
+/**
+ * @brief Remove property from confidct and set rd_kafka_conf with its value
+ *
+ * @param vo The property value object
+ *
+ * @returns 1 on success or 0 on failure (exception raised).
+ */
+static int common_conf_set_special(PyObject *confdict, rd_kafka_conf_t *conf,
+                                   const char *name, PyObject *vo) {
+        const char *v;
+        char errstr[256];
+        PyObject *vs;
+        PyObject *vs8 = NULL;
+
+        if (!(vs = cfl_PyObject_Unistr(vo))) {
+                PyErr_Format(PyExc_TypeError, "expected configuration property %s "
+                             "as type unicode string", name);
+                return 0;
+        }
+
+        v = cfl_PyUnistr_AsUTF8(vs, &vs8);
+        if (rd_kafka_conf_set(conf, name, v, errstr, sizeof(errstr))
+            != RD_KAFKA_CONF_OK) {
+                cfl_PyErr_Format(RD_KAFKA_RESP_ERR__INVALID_ARG,
+                                 "%s", errstr);
+                Py_DECREF(vs);
+                Py_XDECREF(vs8);
+                return 0;
+        }
+
+        Py_DECREF(vs);
+        Py_XDECREF(vs8);
+        PyDict_DelItemString(confdict, name);
+        return 1;
+}
 
 /**
  * Common config setup for Kafka client handles.
@@ -1507,7 +1528,6 @@ rd_kafka_conf_t *common_conf_setup (rd_kafka_type_t ktype,
 				    PyObject *args,
 				    PyObject *kwargs) {
 	rd_kafka_conf_t *conf;
-	rd_kafka_topic_conf_t *tconf;
 	Py_ssize_t pos = 0;
 	PyObject *ko, *vo;
         PyObject *confdict = NULL;
@@ -1560,62 +1580,55 @@ rd_kafka_conf_t *common_conf_setup (rd_kafka_type_t ktype,
         }
 
 	conf = rd_kafka_conf_new();
-	tconf = rd_kafka_topic_conf_new();
 
         /*
-         * Default config (overridable by user)
+         * Set debug contexts first to capture all events including plugin loading
          */
-
-        /* Enable valid offsets in delivery reports */
-        rd_kafka_topic_conf_set(tconf, "produce.offset.report", "true", NULL, 0);
+         if ((vo = PyDict_GetItemString(confdict, "debug")) &&
+              !common_conf_set_special(confdict, conf, "debug", vo))
+                        goto outer_err;
 
         /*
-         * Plugins must be configured prior to handling any of their configuration properties.
-         * Dicts are unordered so we explicitly check for, set, and delete the plugin paths here.
-         * This ensures plugin configuration properties are handled in the correct order.
+         * Plugins must be configured prior to handling any of their
+         * configuration properties.
+         * Dicts are unordered so we explicitly check for, set, and delete the
+         * plugin paths here.
+         * This ensures plugin configuration properties are handled in the
+         * correct order.
          */
         if ((vo = PyDict_GetItemString(confdict, "plugin.library.paths"))) {
-                const char *v;
-                char errstr[256];
-                PyObject *vs = NULL, *vs8 = NULL;
+                /* Resolve plugin paths */
+                PyObject *resolved;
 
-                if (!(vs = cfl_PyObject_Unistr(vo))) {
-                        PyErr_SetString(PyExc_TypeError,
-                                "expected configuration property name "
-                                "as type unicode string");
-                        rd_kafka_topic_conf_destroy(tconf);
-                        rd_kafka_conf_destroy(conf);
-                        Py_DECREF(confdict);
+                resolved = resolve_plugins(vo);
+                if (!resolved)
+                        goto outer_err;
 
-                        return NULL;
+                if (!common_conf_set_special(confdict, conf,
+                                             "plugin.library.paths",
+                                             resolved)) {
+                        Py_DECREF(resolved);
+                        goto outer_err;
                 }
+                Py_DECREF(resolved);
+        }
 
-                v = cfl_PyUnistr_AsUTF8(vs, &vs8);
-
-                if (rd_kafka_conf_set(conf, "plugin.library.paths", v, errstr, sizeof(errstr))
-                   != RD_KAFKA_CONF_OK) {
-                        cfl_PyErr_Format(RD_KAFKA_RESP_ERR__INVALID_ARG,
-                                "%s", errstr);
-
-                        rd_kafka_topic_conf_destroy(tconf);
-                        rd_kafka_conf_destroy(conf);
-                        Py_DECREF(confdict);
-
-                        Py_XDECREF(vs8);
-                        Py_XDECREF(vs);
-
-                        return NULL;
+        if ((vo = PyDict_GetItemString(confdict, "default.topic.config"))) {
+        /* TODO: uncomment for 1.0 release
+                PyErr_Warn(PyExc_DeprecationWarning,
+                             "default.topic.config has being deprecated, "
+                             "set default topic configuration values in the global dict");
+        */
+                if (PyDict_Update(confdict, vo) == -1) {
+                        goto outer_err;
                 }
-
-                Py_XDECREF(vs8);
-                Py_DECREF(vs);
-
-                PyDict_DelItemString(confdict, "plugin.library.paths");
+                PyDict_DelItemString(confdict, "default.topic.config");
         }
 
 	/* Convert config dict to config key-value pairs. */
 	while (PyDict_Next(confdict, &pos, &ko, &vo)) {
-		PyObject *ks, *ks8;
+		PyObject *ks;
+		PyObject *ks8 = NULL;
 		PyObject *vs = NULL, *vs8 = NULL;
 		const char *k;
 		const char *v;
@@ -1623,42 +1636,19 @@ rd_kafka_conf_t *common_conf_setup (rd_kafka_type_t ktype,
                 int r = 0;
 
 		if (!(ks = cfl_PyObject_Unistr(ko))) {
-			PyErr_SetString(PyExc_TypeError,
-					"expected configuration property name "
-					"as type unicode string");
-			rd_kafka_topic_conf_destroy(tconf);
-			rd_kafka_conf_destroy(conf);
-                        Py_DECREF(confdict);
-
-			return NULL;
+                        PyErr_SetString(PyExc_TypeError,
+                                        "expected configuration property name "
+                                        "as type unicode string");
+                        goto inner_err;
 		}
 
 		k = cfl_PyUnistr_AsUTF8(ks, &ks8);
-		if (!strcmp(k, "default.topic.config")) {
-			if (populate_topic_conf(tconf, k, vo) == -1) {
-				Py_DECREF(ks);
-				rd_kafka_topic_conf_destroy(tconf);
-				rd_kafka_conf_destroy(conf);
-                                Py_DECREF(confdict);
-				return NULL;
-			}
-                        Py_XDECREF(ks8);
-			Py_DECREF(ks);
-			continue;
-
-		} else if (!strcmp(k, "error_cb")) {
+		if (!strcmp(k, "error_cb")) {
 			if (!PyCallable_Check(vo)) {
 				PyErr_SetString(PyExc_TypeError,
 						"expected error_cb property "
 						"as a callable function");
-				rd_kafka_topic_conf_destroy(tconf);
-				rd_kafka_conf_destroy(conf);
-                                Py_DECREF(confdict);
-
-                                Py_XDECREF(ks8);
-				Py_DECREF(ks);
-
-				return NULL;
+                                goto inner_err;
                         }
 			if (h->error_cb) {
 				Py_DECREF(h->error_cb);
@@ -1676,14 +1666,7 @@ rd_kafka_conf_t *common_conf_setup (rd_kafka_type_t ktype,
                                 PyErr_SetString(PyExc_ValueError,
                                         "expected throttle_cb property "
                                         "as a callable function");
-                                rd_kafka_topic_conf_destroy(tconf);
-                                rd_kafka_conf_destroy(conf);
-                                Py_DECREF(confdict);
-
-                                Py_XDECREF(ks8);
-                                Py_DECREF(ks);
-
-                                return NULL;
+                                goto inner_err;
                         }
                         if (h->throttle_cb) {
                                 Py_DECREF(h->throttle_cb);
@@ -1701,14 +1684,7 @@ rd_kafka_conf_t *common_conf_setup (rd_kafka_type_t ktype,
 				PyErr_SetString(PyExc_TypeError,
 						"expected stats_cb property "
 						"as a callable function");
-				rd_kafka_topic_conf_destroy(tconf);
-				rd_kafka_conf_destroy(conf);
-                                Py_DECREF(confdict);
-
-                                Py_XDECREF(ks8);
-				Py_DECREF(ks);
-
-				return NULL;
+                                goto inner_err;
                         }
 
 			if (h->stats_cb) {
@@ -1739,19 +1715,12 @@ rd_kafka_conf_t *common_conf_setup (rd_kafka_type_t ktype,
 
 		/* Special handling for certain config keys. */
 		if (ktype == RD_KAFKA_PRODUCER)
-			r = producer_conf_set_special(h, conf, tconf, k, vo);
+			r = producer_conf_set_special(h, conf, k, vo);
 		else if (ktype == RD_KAFKA_CONSUMER)
-			r = consumer_conf_set_special(h, conf, tconf, k, vo);
+			r = consumer_conf_set_special(h, conf, k, vo);
 		if (r == -1) {
 			/* Error */
-                        Py_XDECREF(ks8);
-			Py_DECREF(ks);
-			rd_kafka_topic_conf_destroy(tconf);
-			rd_kafka_conf_destroy(conf);
-                        Py_DECREF(confdict);
-
-			return NULL;
-
+                        goto inner_err;
 		} else if (r == 1) {
 			/* Handled */
 			continue;
@@ -1769,14 +1738,7 @@ rd_kafka_conf_t *common_conf_setup (rd_kafka_type_t ktype,
                                                 "expected configuration "
                                                 "property value as type "
                                                 "unicode string");
-                                rd_kafka_topic_conf_destroy(tconf);
-                                rd_kafka_conf_destroy(conf);
-                                Py_DECREF(confdict);
-
-                                Py_XDECREF(ks8);
-                                Py_DECREF(ks);
-
-                                return NULL;
+                                goto inner_err;
                         }
                         v = cfl_PyUnistr_AsUTF8(vs, &vs8);
                 }
@@ -1785,23 +1747,22 @@ rd_kafka_conf_t *common_conf_setup (rd_kafka_type_t ktype,
 		    RD_KAFKA_CONF_OK) {
 			cfl_PyErr_Format(RD_KAFKA_RESP_ERR__INVALID_ARG,
 					  "%s", errstr);
-			rd_kafka_topic_conf_destroy(tconf);
-			rd_kafka_conf_destroy(conf);
-                        Py_DECREF(confdict);
-
-                        Py_XDECREF(vs8);
-                        Py_XDECREF(vs);
-                        Py_XDECREF(ks8);
-			Py_DECREF(ks);
-
-			return NULL;
+                        goto inner_err;
 		}
 
                 Py_XDECREF(vs8);
                 Py_XDECREF(vs);
                 Py_XDECREF(ks8);
 		Py_DECREF(ks);
-	}
+                continue;
+
+inner_err:
+                Py_XDECREF(vs8);
+                Py_XDECREF(vs);
+                Py_XDECREF(ks8);
+                Py_XDECREF(ks);
+                goto outer_err;
+        }
 
         Py_DECREF(confdict);
 
@@ -1821,9 +1782,6 @@ rd_kafka_conf_t *common_conf_setup (rd_kafka_type_t ktype,
                 rd_kafka_conf_set_log_cb(conf, log_cb);
         }
 
-	rd_kafka_topic_conf_set_opaque(tconf, h);
-	rd_kafka_conf_set_default_topic_conf(conf, tconf);
-
 	rd_kafka_conf_set_opaque(conf, h);
 
 #ifdef WITH_PY_TSS
@@ -1840,6 +1798,12 @@ rd_kafka_conf_t *common_conf_setup (rd_kafka_type_t ktype,
         h->initiated = 1;
 
 	return conf;
+
+outer_err:
+        Py_DECREF(confdict);
+        rd_kafka_conf_destroy(conf);
+
+        return NULL;
 }
 
 
@@ -2156,7 +2120,7 @@ static PyObject *libversion (PyObject *self, PyObject *args) {
 }
 
 static PyObject *version (PyObject *self, PyObject *args) {
-	return Py_BuildValue("si", "0.11.5", 0x000b0500);
+	return Py_BuildValue("si", "0.11.6", 0x000b0600);
 }
 
 static PyMethodDef cimpl_methods[] = {
